#
# Anbox - The Android in a Box runtime environment
# Copyright 2018 Canonical Ltd.  All rights reserved.
#

from charmhelpers.core import hookenv

from charms.reactive import Endpoint
from charms.reactive import when, when_not
from charms.reactive import set_flag, clear_flag


class RestProvides(Endpoint):
    @when('endpoint.{endpoint_name}.joined')
    def joined(self):
        set_flag(self.expand_name('endpoint.{endpoint_name}.available'))

    @when_not('endpoint.{endpoint_name}.joined')
    def departed(self):
        clear_flag(self.expand_name('endpoint.{endpoint_name}.available'))

    def publish_info(self, **kwargs):
        """
        Publish the data over the relationship so that it is
        accessible to the remote units at the other side
        """
        for relation in self.relations:
            if kwargs is not None:
                for k, v in kwargs.items():
                    relation.to_publish[k] = v

            if 'node' not in relation.to_publish.keys():
                relation.to_publish['node'] = hookenv.local_unit()

            if 'private_address' not in relation.to_publish.keys():
                relation.to_publish['private_address'] = (
                    hookenv.unit_private_ip()
                )

            if 'public_address' not in relation.to_publish.keys():
                relation.to_publish['public_address'] = (
                    hookenv.unit_public_ip()
                )

    def services(self):
        services = []
        for relation in self.relations:
            for unit in relation.units:
                services.append(unit.received)
        return services
