#
# Anbox - The Android in a Box runtime environment
# Copyright 2018 Canonical Ltd.  All rights reserved.
#

from charms.reactive import Endpoint
from charms.reactive import set_flag, clear_flag
from charms.reactive import when, when_not


class RestRequires(Endpoint):
    # {endpoint_name} will be filled in by the reactive framework. This is the
    # name of the endpoint as defined in `metadata.yaml`
    @when('endpoint.{endpoint_name}.changed')
    def changed(self):
        set_flag(self.expand_name('endpoint.{endpoint_name}.available'))
        clear_flag(self.expand_name('endpoint.{endpoint_name}.changed'))
        self._notify_changed()

    @when_not('endpoint.{endpoint_name}.joined')
    def broken(self):
        clear_flag(self.expand_name('endpoint.{endpoint_name}.available'))
        self._notify_changed()

    @when('endpoint.{endpoint_name}.departed')
    def departed(self):
        self._notify_changed()
        clear_flag(self.expand_name('endpoint.{endpoint_name}.departed'))

    def _notify_changed(self):
        # We still have to indicate to the rest of the code that the relation
        # has changed and needs processing. Using the possible already set flag
        # .available doesn't work as the handler will be then executed even when
        # nothing has changed. We introduce a new changed flag here to represent
        # the change and the handler must clear the flag itself
        set_flag(self.expand_name('{endpoint_name}.changed'))

    def services(self):
        services = []
        for relation in self.relations:
            for unit in relation.units:
                services.append(unit.received)
        return services

    def publish_info(self, **kwargs):
        """
        Publish the data over the relationship so that it is
        accessible to the remote units at the other side
        """
        for relation in self.relations:
            if kwargs is not None:
                for k, v in kwargs.items():
                    relation.to_publish[k] = v
