#
# Copyright 2021 Canonical Ltd.  All rights reserved.
#

import http.client
import requests
import subprocess

AWS_METADATA_SERVICE_ADDRESS = '169.254.169.254'
AWS_METADATA_SERVICE_BASE_URL = 'http://{}/latest/meta-data'.format(
    AWS_METADATA_SERVICE_ADDRESS)


def retrieve_public_address():
    # There may be situations where public address is not set
    # but they do exist, such as using manual providers on top
    # of the public cloud
    rsp = requests.get(AWS_METADATA_SERVICE_BASE_URL + "/public-ipv4")
    # Possible errors described on AWS docs:
    # 400, 401, 403, 404
    if rsp.status_code >= 400:
        return ""
    return rsp.text


def is_aws_cloud():
    """
    Returns True if the charm is running in an AWS deployment.
    WARNING: in some cases, this function can take a few seconds to return, avoid
    calling it too often and cache the result
    """
    output = subprocess.call('uname -r | grep -q aws', shell=True)
    if output != 0:
        return False

    # Checking for the kernel is not enough as we may use the AWS kernel in our CI
    # We check for the instance metadata endpoint which is only available on AWS but incurs
    # a network overhead
    try:
        conn = http.client.HTTPConnection(AWS_METADATA_SERVICE_ADDRESS, 80, timeout=5)
        conn.request("GET", "/")
    except OSError as e:
        return False
    return True